#!/bin/bash

LDAP_SYNC_OUT=/tmp/ldap_slave_sync.out

__MONIT(){
	case "$1" in
		check) [ "$(monit -c /etc/monit/monitrc status | grep -A 1 $2 | grep 'not monitored')" ] && echo "OFF" ;;
		start) monit monitor -c /etc/monit/monitrc $2 ;;
		stop) monit unmonitor -c /etc/monit/monitrc $2 ;;
	esac
}

timestamp() {
    date +"%s"
}

# enable monitoring and disable ldap memory mod.
_doExit(){
    printf "state:%s;users_expected:%d;users_actual:%d;" "completed" "$USERS_1" "$USERS_2" > $LDAP_SYNC_OUT

    kill $PID_PROVIDER_DISABLE_MONITORING
    wait $PID_PROVIDER_DISABLE_MONITORING 2>/dev/null
    __MONIT start ldap
    __MONIT start sipproxy
    /usr/sbin/fix_ldap_slave "enable"
    /etc/init.d/slapd restart
    /etc/init.d/callweaver restart
    exit 0
}

_toLog(){
    msg="$1"
    logger -t "monitor_replication" -i "$$" -p local0.info "$msg"
}

provideDisableMonitoringOfLdapAndSipProxy() {
    while true; do
        if [ "$(__MONIT check ldap)" != "OFF" ]; then
            __MONIT stop ldap
            __MONIT stop sipproxy
        fi

        sleep 3
    done
}

isAdminSynced() {
    local local_uid=$(slapcat -a '(&(objectClass=wPerson)(cn=admin)(userKind=admin))' | awk -F ': ' '$1 == "entryUUID" { print $2; }')
    local remote_uid=$(timeout 30 ldapsearch -xLLL -h 5.15.0.1 -D "cn=admin,dc=wildix" -b "dc=wildix" -w "wildix" "(&(objectClass=wPerson)(cn=admin)(userKind=admin))" entryUUID|grep -v dn| awk "{print \$2}"|head -n1)

    [ "$local_uid" = "$remote_uid" ] && echo yes
}

printf "state:%s;users_expected:%d;users_actual:%d;" "in progress" 0 0 > $LDAP_SYNC_OUT

provideDisableMonitoringOfLdapAndSipProxy &
PID_PROVIDER_DISABLE_MONITORING=$!

/usr/sbin/fix_ldap_slave "disable"
sleep 5

SUCCESS_ATTEMPTS=0
MASTER_READ_FAILS=0
MASTER_PARTIAL_MATCHES=0
ADMIN_SYNC_FAILS=0
START_TIME=$(timestamp)
_toLog "LDAP Replication started at: $START_TIME"

while true
do
    CURR_TIME=$(timestamp)
    if [ $(($CURR_TIME - $START_TIME)) -gt $((6 * 60 * 60 * 1)) ]; then
        _toLog "Replication timeout 6 hours reached and exiting"
        _doExit
    fi

    USERS_1="$(slapcat | grep -c 'dn: uid=')"
    sleep 5
    USERS_2="$(slapcat | grep -c 'dn: uid=')"

    _toLog "Users count: $USERS_2"
    printf "state:%s;users_expected:%d;users_actual:%d;" "in progress" "$USERS_1" "$USERS_2" > $LDAP_SYNC_OUT

    if [ "$USERS_1" -eq "$USERS_2" ]; then
        ((SUCCESS_ATTEMPTS++))
        _toLog "Replication stop. Check # $SUCCESS_ATTEMPTS"

        if [ $SUCCESS_ATTEMPTS -eq 5 ]; then
            # when replication stopped, checking is admin synced
            if [ "$(isAdminSynced)" != 'yes' ]; then
                # admin is not synced stile
                SUCCESS_ATTEMPTS=0
                ((ADMIN_SYNC_FAILS++))

                if [ $ADMIN_SYNC_FAILS -eq 5 ]; then
                    _toLog "Failed to sync admin. Exiting"
                    _doExit
                fi

                continue
            fi

            # when replication stopped, read users's count form master
            MASTER_USERS_MIDDLE="$(timeout 60 ldapsearch -xLLL -h 5.15.0.1 -D 'cn=admin,dc=wildix' -b 'dc=wildix' -w 'wildix' "(objectClass=wPerson)" dn | grep -c 'dn: uid=')"
            _toLog "Users count on Primary PBX: $MASTER_USERS_MIDDLE"
            if [ "$MASTER_USERS_MIDDLE" -eq "$USERS_2" ]; then
                _toLog "Replication stopped with 100% match and exiting"
                _doExit
            elif [ $MASTER_USERS_MIDDLE -eq 0 ]; then
                # master somehow is not available, wait if it's available
                SUCCESS_ATTEMPTS=0
                ((MASTER_READ_FAILS++))
                if [ $MASTER_READ_FAILS -eq 5 ]; then
                    _toLog "Failed to read from master. Exiting"
                    _doExit
                fi
                sleep 30
            else
                SUCCESS_ATTEMPTS=0
                MASTER_READ_FAILS=0
                ((MASTER_PARTIAL_MATCHES++))
                if [ $MASTER_PARTIAL_MATCHES -eq 5 ]; then
                    # master is not available for too long
                    _toLog "Replication stopped with partial match and exiting"
                    _doExit
                fi
                sleep 30
            fi
        fi
    elif [ $SUCCESS_ATTEMPTS -gt 0 ]; then
        _toLog "Replication continued after check # $SUCCESS_ATTEMPTS"
        SUCCESS_ATTEMPTS=0
        MASTER_PARTIAL_MATCHES=0
    fi
done
