(function($){ // To-do: remove after prototype removed
    var LanguageSettings = function(){
        this.constructor();
    };

    LanguageSettings.prototype = {
        constructor: function(data){
            this.langs = {
                'en': 'English',
                'it': 'Italian',
                'fr': 'French',
                'de': 'German',
                'nl': 'Dutch',
                'en-us': 'English-US',
                'ru': 'Russian',
                'ee': 'Estonian'
            };
            this.settings = {};
            this.LARform = $('#lang_and_region_settings_form');

            //
            this.selectArea = this.LARform.find('select[name=area]');
            this.selectZone = this.LARform.find('select[name=zone]');
            this.inputCountryCode = this.LARform.find('input[name=countrycode]');
            this.selectCountryCode = this.inputCountryCode.prev('select');

            // validate forms
            this.LARform.checkForm({
                rules: [
                    'reg_exp,lar_countrycode,^(1[0-9]{0\\,3}|[2-9][0-9]{0\\,2})$,' + WMS._t('rules.enterValidValue'),
                    'digits_only,lar_countrycode,' + WMS._t('rules.enterNumber')
                ]
            });
            //

            this.loadSettings();

            // event
            this.selectArea.on('change', jQuery.proxy(this.setFieldTimezoneZone, this, null));
            this.LARform.find('select[name=language]').on('change', jQuery.proxy(this.onchangeLang, this));
            this.selectCountryCode.on('change', jQuery.proxy(this.onchangeCountryCode, this));
            this.LARform.find('.settings_save').on('click', jQuery.proxy(this.saveLangAndRegionForm, this));
            this.LARform.on('change', 'input[type=checkbox]', jQuery.proxy(this.checkMaxInstall, this));

            $(window).bind('beforeunload', $.proxy(function(){
                if (this.isDirty()){
                    return WMS._t('prefs.are_you_sure_want_to_exit_without_save');
                }
            }, this));
        },
        loadSettings: function(){
            ap_showWaitMessage('waitDiv', 1);

            $.ajax({
                url: 'api/v1/PBX/settings/LanguageAndRegion/',
                type: 'GET',
                dataType: 'json',
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.result){
                        // console.log(response.result);
                        this.settings = response.result;
                        this.setFieldLanguage(this.settings.language, this.settings.lists.langs_for_install);
                        this.setFieldTonezone(this.settings.tonezone, this.settings.lists.tonezones);
                        this.setFieldTimezone(this.settings.timezone, this.settings.lists.timezones);
                        // load country code
                        countryList.addCustom(WMS._t('trunks.customCountry'));
                        this.setFieldCountryCode(this.settings.countrycode, countryList.list);

                        this.createFormLangs(this.settings.lists.langs_for_install);
                        this.setValuesLangs(this.settings.installed_langs);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this.parseResponseError(jqXHR));
                }, this),
                complete: function(){
                    ap_showWaitMessage('waitDiv', 0);
                }
            });
        },
        setFieldLanguage: function(value, list){
            var select = this.LARform.find('select[name=language]');
            select.empty();
            if(list){
                var option;
                $.each(list, function(index, val){
                    option = $('<option>');
                    option.val(val).text(this.langs[val] || val);
                    select.append(option);
                }.bind(this));
            }
            if(value){
                select.val(value);
            }
        },
        setFieldTonezone: function(value, list){
            var select = this.LARform.find('select[name=tonezone]');
            select.empty();
            select.append($('<option>'));
            if(list){
                var option;
                $.each(list, function(index, val){
                    option = $('<option>');
                    option.val(index).text(val);
                    select.append(option);
                });
            }
            if(value){
                select.val(value);
            }
        },
        setFieldTimezone: function(value, list){
            value = value || {};
            var option;
            this.selectArea.empty();
            if(list){
                $.each(list, function(index){
                    option = $('<option>');
                    option.val(index).text(index);
                    this.selectArea.append(option);
                }.bind(this));
                this.selectArea.val(value.area);
            }
            this.setFieldTimezoneZone(value.zone);
        },
        setFieldTimezoneZone: function(value){
            this.selectZone.empty();
            var area = this.selectArea.val();
            if(area){
                var listZone = this.settings.lists.timezones[area];
                if(listZone){
                    $.each(listZone, function(index, val){
                        var option = $('<option>');
                        option.val(val).text(val);
                        this.selectZone.append(option);
                    }.bind(this));
                    if(value){
                        this.selectZone.val(value);
                    }
                }
            }
        },
        setFieldCountryCode: function(value, list){
            this.selectCountryCode.empty();
            var custom = true;
            if(list){
                $.each(list, function(index, val){
                    var option = $('<option>');
                    option.val(val.code).text((val.code == 0 ? '' : val.code + ' ') + val.name);
                    this.selectCountryCode.append(option);
                    if(val.code == value){
                        custom = false;
                    }
                }.bind(this));
                if(value){
                    if(custom){
                        this.selectCountryCode.val(0);
                        this.onchangeCountryCode();
                    }else{
                        this.selectCountryCode.val(value);
                    }
                    this.inputCountryCode.val(value);
                }
            }
        },
        onchangeCountryCode: function(){
            if(this.selectCountryCode.val() == 0){
                this.selectCountryCode.width('182px');
                this.inputCountryCode.show();
            }else{
                this.inputCountryCode.hide();
                this.inputCountryCode.val(this.selectCountryCode.val());
            }
        },
        onchangeLang: function(){
            var val = this.LARform.find('select[name=language]').val();
            this.LARform.find('input[type=checkbox]:not([value=en]):checked').attr({'disabled': false});
            this.LARform.find('input.value').attr({'checked': false}).removeClass('value');
            if(val != 'en'){
                var el = this.LARform.find('input[value=' + val + ']');
                if(!el.is(':checked')){
                    el.attr({'checked': true, 'disabled': true}).addClass('value');
                }else{
                    el.attr('disabled', true);
                }
            }
            this.checkMaxInstall();
        },
        getData: function(){
            var data = {
                language: this.LARform.find('select[name=language]').val(),
                tonezone: this.LARform.find('select[name=tonezone]').val(),
                area: this.selectArea.val(),
                zone: this.selectZone.val(),
                countrycode: this.inputCountryCode.val(),
                lang: []
            };
            this.LARform.find('input[type=checkbox]:checked').each(function(index, el){
                data.lang.push($(el).val());
            });
            return data;
        },
        setData: function(data){
            this.settings.language = data.language;
            this.settings.tonezone = data.tonezone;
            this.settings.timezone = {area: data.area, zone: data.zone};
            this.settings.countrycode = data.countrycode;
            this.settings.installed_langs = data.lang;
        },
        saveLangAndRegionForm: function(){
            var data = this.getData();
            if(!this.isDirty(data)){
                return false;
            }
            if(!this.LARform.checkForm('validate')){
                return false;
            }
            if(this.LARform.find('input[type=checkbox]:checked').length == 0){
                return setError('error', 'Must be selected at least one language', 5000);
            }
            if(data.lang.length > this.settings.max_install){
                $('.spacememory').fadeOut('slow', function() {
                    $(this).fadeIn('slow', function() {
                        $(this).fadeOut('slow', function() {
                            $(this).fadeIn('slow');
                        });
                    });
                });
                return false;
            }

            ap_showWaitMessage('waitDiv', 1);
            $.ajax({
                url: 'api/v1/PBX/settings/LanguageAndRegion/',
                type: 'POST',
                dataType: 'json',
                data: data,
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.type == 'result'){
                        if(!this.isDirty({lang: data.lang})){
                            setError('success', response.result);
                            ap_showWaitMessage('waitDiv', 0);
                        }else{
                            this.saveStatus();
                        }

                        this.setData(data);
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this.parseResponseError(jqXHR));
                    ap_showWaitMessage('waitDiv', 0);
                }, this)
            });
            return true;
        },
        saveStatus: function(){
            $.ajax({
                url: 'api/v1/PBX/settings/LanguageAndRegion/status',
                type: 'GET',
                dataType: 'json',
                success: $.proxy(function (response, textStatus, jqXHR) {
                    if (response && response.type == 'result'){
                        if (response.result.processes){
                            setTimeout($.proxy(this.saveStatus, this), 1000);
                        } else {
                            setError('success', response.result);
                            ap_showWaitMessage('waitDiv', 0);
                        }
                    }
                }, this),
                error: $.proxy(function(jqXHR, textStatus, errorThrown){
                    setError('error', this.parseResponseError(jqXHR));
                    ap_showWaitMessage('waitDiv', 0);
                }, this)
            });
        },

        createFormLangs: function(list){
            if(list.length > this.settings.max_install){
                var head = $('.table_sounds').prev('.page_content_head');
                head.append($('<span>').text(' (select max ' + this.settings.max_install + ')').addClass('error spacememory'));
            }
            var tbody = $('.table_sounds').find('tbody');
            var tr = [];
            $.each(list, function(index, val){
                tr.push(
                    $('<tr>')
                        .append($('<th>').append($('<label>').attr({'for': 'pack_' + val}).text(this.langs[val] || val)))
                        .append(
                            $('<td>').append(
                                $('<input/>').attr({type: 'checkbox', name: 'lang[]', id: 'pack_' + val}).val(val)
                            )
                        )
                );
            }.bind(this));

            tbody.prepend(tr);
            tbody.find('input[value=en]').attr('disabled', true); // default lang, not delete
            tbody.find('input[value=' + this.settings.language + ']').attr('disabled', true);
        },
        setValuesLangs: function(value){
            this.LARform.find('input[type=checkbox]').attr('checked', false);
            for(var i = 0; i < value.length; i++){
                this.LARform.find('input[value=' + value[i] + ']').attr('checked', true);
            }
            this.checkMaxInstall();
        },
        checkMaxInstall: function(){
            var notSel = this.LARform.find('input[type=checkbox]:not(:checked)');
            if(this.LARform.find('input[type=checkbox]:checked').length >= this.settings.max_install){
                notSel.attr({'disabled': true, 'title': WMS._t('strings.NoSpaceLeft')}).addClass('notspace');
            }else{
                notSel.attr({'disabled': false, 'title': ''}).removeClass('notspace');
            }
        },
        isDirty: function(data){
            data = data || this.getData();
            var result = false;

            $.each(data, $.proxy(function(name, value){
                if(this.settings[name] === undefined){
                    this.settings[name] = '';
                }
                if(name == 'lang'){
                    if($(this.settings.installed_langs).not(value).get().length){
                        result = true;
                        return false;
                    }else if($(value).not(this.settings.installed_langs).get().length){
                        result = true;
                        return false;
                    }
                }else if(name == 'area' || name == 'zone'){
                    if(this.settings.timezone[name] != value){
                        result = true;
                        return false;
                    }
                }else if(this.settings[name] != value){
                    result = true;
                    return false;
                }
            }, this));
            return result;
        },

        parseResponseError: function(jqXHR){
            var error = '';
            if(jqXHR && jqXHR.responseText && jqXHR.responseText.indexOf('{') === 0){
                var response = jQuery.parseJSON(jqXHR.responseText);
                if(response && response.reason){
                    error = String(response.reason);
                }
            }
            return error || 'Unknown error';
        }

    };

    $(document).ready(function() {
        window.languageSettings = new LanguageSettings();
    });
})(jQuery);
